import fs = require('fs');
import path = require('path');
import domain = require("domain");
import request = require('request');
import vm = require('vm');

import resolveUrl = require("resolve-url");

import {MainModule} from "./main-module";

class RecimeError extends Error {
    constructor(message){
        super(message);
    }
};

export class Runnable{
    private options: any;

    constructor(options){
        this.options = options;
    }

    run(data) : Promise<any> {
         return new Promise((resolve, reject)=>{
            const options = this.options;
            fs.readFile(MainModule.path(options), 'utf-8', (err, code)=>{
                if (err){
                    return reject(err);
                }

                // uid
                options.env['UID'] = options.uid;
                options.env['HOME_DIR'] = options.homedir;
                // pass in platform for external modules to take advantage of.
                options.env['PLATFORM'] = options.platform || "web";
                // aws region
                options.env['REGION'] = process.env.AWS_REGION;

                const p = {
                    env : options.env
                }

                let sandbox = {
                    console : console,
                    exports : exports,
                    process : p,
                    require : (module:string)=>{
                       return require(MainModule.normalizedDepsPath(module, options));
                    },
                    url: (p)=>{
                        if (process.env.AWS_LAMBDA_FUNCTION_NAME){
                           const asset = p.replace(/(\/)?assets\//ig, "")
                           return `https://assets.recime.io/${path.normalize(`${options.uid}/${asset}`)}`
                        }
                        return `https://${options.host}/${path.normalize(`${options.uid}/v1/${p}`)}`;
                     },
                    __dirname: `${options.homedir}/${options.uid}`
                };

                const Module = vm.runInNewContext(code, sandbox);

                if (typeof Module === 'undefined'){
                    reject(new RecimeError("Could not resolve the main module."));
                    return;
                }
                // export default or export
                const Bot = Module.Bot || Module;

                if (Bot.constructor.length === 0){
                    reject(new RecimeError("Invalid Bot class definition."));
                    return;
                }

                const bot = new Bot(data);
                const local = domain.create();

                local.on("error", (err)=>{
                    reject(new RecimeError(err.message));
                });

                // support both callback or promise.
                local.run(()=>{
                    let ret = bot.execute((data)=>{
                        resolve(data)
                    });
                    if (ret && ret.then && bot.execute.length === 0){
                        ret.then((data)=>{
                           resolve(data)
                        }, (err)=>{
                            reject(new RecimeError(err.message));
                        });;
                    }
                });
            });
         });
         
    }
}
