import fs = require("fs");
import util = require("util");
import Ajv = require("ajv");

import {Config} from "./config";

const platforms = ['telegram', 'twilio', 'wechat', 'slack', 'slack-event', "viber"];

export class SchemaValidator {
    private schema:any;

    constructor(schema){
        this.schema = schema;
    }

    static findOne(data){
        return new Promise((resolve, reject)=>{
             const validator = new SchemaValidator(Config.schema);

            const telegram = validator.validate(platforms[0], data);
            const twilio = validator.validate(platforms[1], data);
            const wechat = validator.validate(platforms[2], data);
            const slack = validator.validate(platforms[3], data);
            const slackEvent = validator.validate(platforms[4], data);
            const viber = validator.validate(platforms[5], data);

            Promise.all([telegram, twilio, wechat, slack, slackEvent, viber]).then((results)=>{
                const index = results.indexOf(true);
                if (results.indexOf(true) >=0)
                    resolve(platforms[index]);
                else
                    resolve("web"); 
            });
        });
    }

    validate(platform, data){
        return new Promise((resolve, reject)=>{
            resolve(this.validateSchema(this.schema[platform], data));
        });
    }

    validateSchema(schema, data){
        const ajv = new Ajv();
        const validate = ajv.compile(schema);
        return validate(data);
    }
}

