assert = require('assert')
SchemaValidator = require(__dirname + '/../schema-validator').SchemaValidator

describe 'slack', ()->
    describe 'validate()', ()->
        it 'should return true for valid', ()->
            data = 
                type : "url_verification"
                token: "enBr8Mh1BdVeoGp2FvCxCDl8"
                challenge: "9HaRb9H2dEHT6NGiZFoL6GWQ01qBOJV4XuSkv6U6P3ZL92SznTR6"
            
            SchemaValidator.validate(data, 'slack').then (result)->
                assert.equal(result, true)
            , (err)->
                assert.ok(false)

describe 'slack-event', ()->
    describe 'validate()', ()->
        it 'should return true for valid', ()->
            data = 
                token: 'enBr8Mh1BdVeoGp2FvCxCDl8'
                team_id: 'T257411PU'
                api_app_id: 'A580074JH'
                event: 
                    type: 'member_joined_channel'
                    user: 'U57UZ7REY'
                    channel: 'C57TKS2BU'
                    channel_type: 'C'
                    event_ts: '1493800160.199104'
                type: 'event_callback',
                authed_users: [ 'U57UZ7REY' ]
                event_id: 'Ev577XSNFJ'
                event_time: 1493800160
            
            SchemaValidator.validate(data, 'slack-event').then (result)->
                assert.equal(result, true)
            , (err)->
                assert.ok(false)

describe 'wechat', ()->
    describe 'validate()', ()->
        it 'should return true for valid', ()->
            data = 
                ToUserName : 'to'
                FromUserName : 'from'
                CreateTime : '111333222'
                MsgType : 'text'
                Content : 'Hello World!'
                MsgId : '11111'
            
            SchemaValidator.validate(data, 'wechat').then (result)->
                assert.equal(result, true)
            , (err)->
                assert.ok(false)

describe 'telegram', ()->
    describe 'validate()', ()->
        platform = 'telegram'

        it 'should return true for valid', ()->
            data = 
                update_id : 858720751
                message : 
                    message_id : 430
                    from : 
                        id : 159538549
                        last_name : "Doe"
                        first_name : "John"
                        username : "john"
                    chat : 
                        id : 159538549
                        last_name : "Doe"
                        first_name : "Jane"
                        username : "jane"
                        type : "private"
                    date : (new Date).getTime()
                    text : "Hello world!"

            SchemaValidator.validate(data, platform).then (result)->
                assert.equal(result, true)
            , (err)->
                throw err

        it 'should return false for invalid', ()->
            data = 
                update_id : 1111
                message : 
                    text : "Hello world!"
                    
            SchemaValidator.validate(data, platform).then (result)->
                    assert.equal(result, false)
            , (err)->
                throw err

describe 'schema', ()->
    describe 'findOne()', ()->
        it 'should assert a platform', ()->
            data = 
                ToUserName : 'to'
                FromUserName : 'from'
                CreateTime : '111333222'
                MsgType : 'text'
                Content : 'Hello World!'
                MsgId : '11111'
            
            SchemaValidator.findOne(data).then (platform)->
                assert.equal(platform, 'wechat')
            , (err)->
                throw err
